
import os, json
import click
from usgs import api


def get_node(dataset, node):
    """
    .. todo:: Move to more appropriate place in module.
    """
    
    if node is None:
        
        cur_dir = os.path.dirname(os.path.realpath(__file__))
        data_dir = os.path.join(cur_dir, "..", "data")
        dataset_path = os.path.join(data_dir, "datasets.json")
        
        with open(dataset_path, "r") as f:
            datasets = json.loads(f.read())
        
        node = datasets[dataset].upper()
    
    return node


def to_coordinates(bounds):
    xmin, ymin, xmax, ymax = bounds
    
    return [[
        [xmin, ymin],
        [xmin, ymax],
        [xmax, ymax],
        [xmax, ymin],
        [xmin, ymin]
    ]]


def to_geojson_feature(entry):
    bounds = map(float, entry.pop("sceneBounds").split(','))
    
    coordinates = to_coordinates(bounds)
    
    return {
        "type": "Feature",
        "properties": entry,
        "geometry": {
            "type": "Polygon",
            "coordinates": coordinates
        }
    }

api_key_opt = click.option("--api-key", help="API key returned from USGS servers after logging in.", default=None)
node_opt = click.option("--node", help="The node corresponding to the dataset (CWIC, EE, HDDS, LPVS).", default=None)


@click.group()
def usgs():
    pass


@click.command()
@click.argument("username")
@click.argument("password")
def login(username, password):
    api_key = api.login(username, password)
    print(api_key)


@click.command()
def logout():
    print(api.logout())


@click.command()
@click.argument("node")
def datasets(node):
    data = api.datasets(None, node)
    print(json.dumps(data))


@click.command()
@click.argument("dataset")
@click.argument("scene-ids", nargs=-1)
@node_opt
@click.option("--extended", is_flag=True, help="Probe for more metadata.")
@api_key_opt
def metadata(dataset, scene_ids, node, extended, api_key):
    
    node = get_node(dataset, node)
    
    data = api.metadata(dataset, node, scene_ids, extended=extended, api_key=api_key)
    print(json.dumps(data))


@click.command()
@click.argument("dataset")
@node_opt
def dataset_fields(dataset, node):
    node = get_node(dataset, node)
    
    data = api.dataset_fields(dataset, node)
    
    print(json.dumps(data))


@click.command()
@click.argument("dataset")
@node_opt
@click.option("--start-date")
@click.option("--end-date")
@click.option("--longitude")
@click.option("--latitude")
@click.option("--distance", help="Radius - in units of meters - used to search around the specified longitude/latitude.", default=100)
@click.option("--lower-left", nargs=2, help="Longitude/latitude specifying the lower left of the search window")
@click.option("--upper-right", nargs=2, help="Longitude/latitude specifying the lower left of the search window")
@click.option("--where", nargs=2, multiple=True, help="Supply additional search criteria.")
@click.option('--geojson', is_flag=True)
@api_key_opt
def search(dataset, node, start_date, end_date, longitude, latitude, distance, lower_left, upper_right, where, api_key, geojson):
    
    node = get_node(dataset, node)
    
    if where:
        # Query the dataset fields endpoint for queryable fields
        fields = api.dataset_fields(dataset, node)
        
        def format_fieldname(s):
            return ''.join(c for c in s if c.isalnum()).lower()
        
        field_lut = { format_fieldname(field['name']): field['fieldId'] for field in fields }
        where = { field_lut[format_fieldname(k)]: v for k, v in where if format_fieldname(k) in field_lut }
    
    if lower_left:
        lower_left = dict(zip(['longitude', 'latitude'], lower_left))
        upper_right = dict(zip(['longitude', 'latitude'], upper_right))
    
    data = api.search(dataset, node, lat=latitude, lng=longitude, distance=distance, ll=lower_left, ur=upper_right, start_date=start_date, end_date=end_date, where=where, api_key=api_key)
    
    if geojson:
        features = map(to_geojson_feature, data)
        data = { 'type': 'FeatureCollection', 'features': features }
    
    print(json.dumps(data))


@click.command()
@click.argument("dataset")
@click.argument("scene-ids", nargs=-1)
@node_opt
@api_key_opt
def download_options(dataset, scene_ids, node, api_key):
    
    node = get_node(dataset, node)
    
    data = api.download_options(dataset, node, scene_ids)
    print(json.dumps(data))


@click.command()
@click.argument("dataset")
@click.argument("scene_ids", nargs=-1)
@click.option("--product", nargs=1, required=True)
@node_opt
@api_key_opt
def download_url(dataset, scene_ids, product, node, api_key):
    
    node = get_node(dataset, node)
    
    data = api.download(dataset, node, scene_ids, product)
    print(json.dumps(data))


usgs.add_command(login)
usgs.add_command(logout)
usgs.add_command(datasets)
usgs.add_command(dataset_fields, "dataset-fields")
usgs.add_command(metadata)
usgs.add_command(search)
usgs.add_command(download_options, "download-options")
usgs.add_command(download_url, "download-url")




# Template XML requests required by the USGS Inventory Service
# Requesting data like it's 1999

from xml.etree.ElementTree import Element, SubElement, tostring
from usgs import CATALOG_NODES, USGSApiKeyRequiredError, USGSCatalogNodeDoesNotExist, USGSDependencyRequired


def create_root_request():
    root = Element("soapenv:Envelope")
    root.set("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance")
    root.set("xmlns:xsd", "http://www.w3.org/2001/XMLSchema")
    root.set("xmlns:soapenv", "http://schemas.xmlsoap.org/soap/envelope/")
    root.set("xmlns:soap", "http://earthexplorer.usgs.gov/inventory/soap")

    header = SubElement(root, "soapenv:Header")
    body = SubElement(root, "soapenv:Body")
    
    return (root, body)


def create_request_type(parent, request_type):
    el = SubElement(parent, "soap:%s" % request_type)
    el.set("soapenv:encodingStyle", "http://schemas.xmlsoap.org/soap/encoding/")
    return el


def create_api_key_element(parent, api_key):
    el = SubElement(parent, "apiKey")
    el.set("xsi:type", "xsd:string")
    el.text = api_key


def create_node_element(parent, node):
    
    if node not in CATALOG_NODES:
        raise USGSCatalogNodeDoesNotExist("Catalog nodes include %s" % ", ".join(CATALOG_NODES))
    
    el = SubElement(parent, "node")
    el.set("xsi:type", "xsd:string")
    el.text = node


def create_dataset_element(parent, dataset):
    el = SubElement(parent, "datasetName")
    el.set("xsi:type", "xsd:string")
    el.text = dataset


def create_entity_ids_element(parent, entityids):
    
    if isinstance(entityids, str):
        
        el = SubElement(parent, "entityId")
        el.set("xsi:type", "xsd:string")
        el.text = entityids
    
    else:
        
        el = SubElement(parent, "entityIds")
        el.set("xsi:type", "soap:ArrayOfString")
        
        for entityid in entityids:
            child = SubElement(el, "item")
            child.set("xsi:type","xsd:string")
            child.text = entityid


def create_service_class_coordinate(parent, name, latitude=None, longitude=None):
    
    el = SubElement(parent, name)
    el.set("xsi:type", "soap:Service_Class_Coordinate")
    
    lat_el = SubElement(el, "latitude")
    lat_el.set("xsi:type", "xsd:double")
    lat_el.text = str(latitude)
    
    lng_el = SubElement(el, "longitude")
    lng_el.set("xsi:type", "xsd:double")
    lng_el.text = str(longitude)


def clear_bulk_download_order(dataset, node, api_key=None):
    """
    This method is used to clear bulk download order information from the item basket.
    
    :param dataset:
    
    :param node:
    
    :param api_key:
        API key is required.
    """
    
    if api_key is None:
        raise USGSApiKeyRequiredError
    
    root, body = create_root_request()
    
    el = create_request_type(body, "clearBulkDownloadOrder")
    
    create_node_element(el, node)
    create_dataset_element(el, dataset)
    create_api_key_element(el, api_key)
    
    return tostring(root)


def clear_order(dataset, node, api_key=None):
    """
    This method is used to clear order information from the item basket.
    
    :param dataset:
    
    :param node:
    
    :param api_key:
        API key is required.
    """
    
    if api_key is None:
        raise USGSApiKeyRequiredError
    
    root, body = create_root_request()
    
    el = create_request_type(body, "clearOrder")
    
    create_node_element(el, node)
    create_dataset_element(el, dataset)
    create_api_key_element(el, api_key)
    
    return tostring(root)


def datasets(dataset, node, ll=None, ur=None, start_date=None, end_date=None, api_key=None):
    """
    This method is used to find datasets available for searching.
    By passing no parameters except node, all available datasets
    are returned. Additional parameters such as temporal range
    and spatial bounding box can be used to find datasets that 
    provide more specific data. The dataset name parameter can
    be used to limit the results based on matching the supplied
    value against the dataset name with assumed wildcards at the
    beginning and end. All parameters are optional except for 
    the 'node' parameter.
    
    :param dataset:
        Dataset Identifier
    
    :param ll:
        Lower left corner of an AOI bounding box - in decimal form
        Longitude/Latitude dictionary
        
        e.g. { "longitude": 0.0, "latitude": 0.0 }
    
    :param ur:
        Upper right corner of an AOI bounding box - in decimal form
        Longitude/Latitude dictionary
        
        e.g. { "longitude": 0.0, "latitude": 0.0 }
    
    :param start_date:
        Used for searching scene acquisition - will accept anything
        that the PHP strtotime function can understand
    
    :param end_date:
        Used for searching scene acquisition - will accept anything
        that the PHP strtotime function can understand
    
    :param node:
        The requested Catalog
    
    :param api_key:
        API key is not required.
        
    """
    
    root, body = create_root_request()
    
    el = create_request_type(body, "datasets")
    
    create_node_element(el, node)
    if dataset:
        create_dataset_element(el, dataset)
    
    if api_key:
        create_api_key_element(el, api_key)
    
    if ll and ur:
        
        create_service_class_coordinate(el, "lowerLeft", latitude=ll["latitude"], longitude=ll["longitude"])
        create_service_class_coordinate(el, "upperRight", latitude=ur["latitude"], longitude=ur["longitude"])
        
    if start_date:
        
        start_date_el = SubElement(el, "startDate")
        start_date_el.set("xsi:type", "xsd:string")
        start_date_el.text = start_date
        
    if end_date:
        
        end_date_el = SubElement(el, "endDate")
        end_date_el.set("xsi:type", "xsd:string")
        end_date_el.text = end_date
    
    return tostring(root)


def dataset_fields(dataset, node, api_key=None):
    """
    This request is used to return the metadata filter
    fields for the specified dataset. These values can
    be used as additional criteria when submitting search
    and hit queries.
    
    :param dataset:
    
    :param node:
    
    :param api_key:
        API key is not required.
    """
    
    root, body = create_root_request()
    
    el = create_request_type(body, "datasetFields")
    
    create_dataset_element(el, dataset)
    create_node_element(el, node)
    
    if api_key:
        create_api_key_element(el, api_key)
    
    return tostring(root)


def download(dataset, node, entityids, products, api_key=None):
    """
    The use of this request will be to obtain valid data download URLs.
    
    :param dataset:
    
    :param entityIds:
        list
    
    :param products:
        list
    
    :param node:
    
    :param api_key:
        API key is required.
    """
    
    if api_key is None:
        raise USGSApiKeyRequiredError
    
    root, body = create_root_request()
    
    el = create_request_type(body, "download")
    
    create_dataset_element(el, dataset)
    create_node_element(el, node)
    create_api_key_element(el, api_key)
    create_entity_ids_element(el, entityids)
    
    products_el = SubElement(el, "products")
    products_el.set("xsi:type", "soap:ArrayOfString")
    
    for product in products:
        product_el = SubElement(products_el, "item")
        product_el.set("xsi:type","xsd:string")
        product_el.text = product
        
    return tostring(root)


def download_options(dataset, node, entityids, api_key=None):
    """
    The use of the download options request is to discover the different download
    options for each scene. Some download options may exist but still be unavailable
    due to disk usage and many other factors. If a download is unavailable
    it may need to be ordered.
    
    :param dataset:
    
    :param node:
    
    :param entityIds:
    
    :param api_key:
        API key is not required.
    """
    
    root, body = create_root_request()
    
    el = create_request_type(body, "downloadOptions")
    
    create_dataset_element(el, dataset)
    create_node_element(el, node)
    
    if api_key:
        create_api_key_element(el, api_key)
    
    create_entity_ids_element(el, entityids)
    
    return tostring(root)


def get_bulk_download_products(dataset, node, entityids, api_key):
    """
    Retrieve bulk download products on a scene-by-scene basis.
    
    :param dataset:
    
    :param node:
    
    :param entityid:
        String or list of strings.
    
    :param api_key:
        API key is required.
        
    """
    
    if api_key is None:
        raise USGSApiKeyRequiredError
    
    root, body = create_root_request()
    
    el = create_request_type(body, "getBulkDownloadProducts")
    
    create_dataset_element(el, dataset)
    create_node_element(el, node)
    create_api_key_element(el, api_key)
    create_entity_ids_element(el, entityids)
    
    return tostring(root)


def get_order_products(dataset, node, entityids, api_key=None):
    """
    Retrieve orderable products on a scene-by-scene basis.
    
    :param dataset:
    
    :param node:
    
    :param entityid:
    
    :param api_key:
        API key is required.
    
    .. todo:: Support multiple scene request.
    """
    
    if api_key is None:
        raise USGSApiKeyRequiredError
    
    root, body = create_root_request()
    
    el = create_request_type(body, "getOrderProducts")
    
    create_dataset_element(el, dataset)
    create_node_element(el, node)
    create_api_key_element(el, api_key)
    create_entity_ids_element(el, entityids)
    
    return tostring(root)


def item_basket(api_key=None):
    """
    Returns the current item basket for the current user.
    
    :param api_key:
        API key is required.
    """
    
    if api_key is None:
        raise USGSApiKeyRequiredError
    
    root, body = create_root_request()
    
    el = create_request_type(body, "itemBasket")
    create_api_key_element(el, api_key)
    
    return tostring(root)


def login(username, password):
    """
    This method requires SSL be used due to the sensitive nature of
    users passwords. Upon a successful login, an API key will be
    returned. This key will be active for one hour and should be
    destroyed upon final use of the service by calling the logout
    method. Users must have "Machine to Machine" access based on
    a user-based role in the users profile.
    
    :param username:
    
    :param password:
    """
    
    root, body = create_root_request()
    
    el = create_request_type(body, "login")
    
    username_el = SubElement(el, "username")
    username_el.set("xsi:type", "xsd:string")
    username_el.text = username
    
    password_el = SubElement(el, "password")
    password_el.set("xsi:type", "xsd:string")
    password_el.text = password
    
    return tostring(root)


def logout(api_key=None):
    """
    Remove the users API key from being used in the future.
    
    :param api_key:
        API key is required.
    """
    
    root, body = create_root_request()
    
    el = create_request_type(body, "logout")
    create_api_key_element(el, api_key)
    
    return tostring(root)


def metadata(dataset, node, sceneids, api_key=None):
    """
    The use of the metadata request is intended for those who have
    acquired scene IDs from a different source. It will return the
    same metadata that is available via the search request.
    
    :param dataset:
    
    :param node:
    
    :param sceneid:
    
    :param api_key:
    """
    root, body = create_root_request()
    
    el = create_request_type(body, "metadata")
    
    create_dataset_element(el, dataset)
    create_node_element(el, node)
    
    if api_key:
        create_api_key_element(el, api_key)
    
    create_entity_ids_element(el, sceneids)
    
    return tostring(root)


def remove_bulk_download_scene():
    raise NotImplementedError


def remove_order_scene():
    raise NotImplementedError


def search(dataset, node, lat=None, lng=None, distance=100, ll=None, ur=None, start_date=None, end_date=None, where=None, max_results=50000, starting_number=1, sort_order="DESC", api_key=None):
    """
    :param dataset:
    
    :param node:
    
    :param lat:
    
    :param lng:
    
    :param ll:
    
    :param distance:
    
    :param ur:
    
    :param start_date:
    
    :param end_date:
    
    :param where:
        Specify additional search criteria
    
    :param max_results:
    
    :param starting_number:
    
    :param sort_order:
    
    :param api_key:
        API key is not required.
    """
    root, body = create_root_request()
    
    el = create_request_type(body, "search")
    
    create_dataset_element(el, dataset)
    create_node_element(el, node)
    
    # Latitude and longitude take precedence over ll and ur
    if lat and lng:
        
        try:
            import pyproj
            from shapely import geometry
        except ImportError:
            raise USGSDependencyRequired("Shapely and PyProj are required for spatial searches.")
        
        prj = pyproj.Proj(proj='aeqd', lat_0=lat, lon_0=lng)
        half_distance = 0.5 * distance
        box = geometry.box(-half_distance, -half_distance, half_distance, half_distance)
        
        lngs, lats = prj(*box.exterior.xy, inverse=True)
        
        ll = { "longitude": min(*lngs), "latitude": min(*lats) }
        ur = { "longitude": max(*lngs), "latitude": max(*lats) }
    
    if ll and ur:
        
        create_service_class_coordinate(el, "lowerLeft", latitude=ll["latitude"], longitude=ll["longitude"])
        create_service_class_coordinate(el, "upperRight", latitude=ur["latitude"], longitude=ur["longitude"])
        
    if start_date:
        
        start_date_el = SubElement(el, "startDate")
        start_date_el.set("xsi:type", "xsd:string")
        start_date_el.text = start_date
        
    if end_date:
        
        end_date_el = SubElement(el, "endDate")
        end_date_el.set("xsi:type", "xsd:string")
        end_date_el.text = end_date
    
    if where:
        
        # TODO: Support more than AND key/value equality queries
        additional_criteria_el = SubElement(el, "additionalCriteria")
        
        filter_type_el = SubElement(additional_criteria_el, "filterType")
        filter_type_el.text = "and"
        
        child_filters_el = SubElement(additional_criteria_el, "childFilters")
        for field_id, value in where.iteritems():
            child_item_el = SubElement(child_filters_el, "item")
            field_id_el = SubElement(child_item_el, "fieldId")
            field_id_el.text = str(field_id)
            item_filter_type_el = SubElement(child_item_el, "filterType")
            item_filter_type_el.text = "value"
            operand_el = SubElement(child_item_el, "operand")
            operand_el.text = "="
            value_el = SubElement(child_item_el, "value")
            value_el.text = str(value)

    if max_results:
        
        max_results_el = SubElement(el, "maxResults")
        max_results_el.set("xsi:type", "xsd:int")
        max_results_el.text = str(max_results)
        
    if starting_number:
        
        starting_number_el = SubElement(el, "startingNumber")
        starting_number_el.set("xsi:type", "xsd:int")
        starting_number_el.text = str(starting_number)
        
    if sort_order:
        
        sort_order_el = SubElement(el, "sortOrder")
        sort_order_el.set("xsi:type", "xsd:string")
        sort_order_el.text = sort_order
    
    if api_key:
        create_api_key_element(el, api_key)
    
    return tostring(root)


def submit_bulk_order():
    raise NotImplementedError


def submit_order():
    raise NotImplementedError


def update_bulk_download_scene():
    raise NotImplementedError


def update_order_scene():
    raise NotImplementedError



from codecs import open as codecs_open
from setuptools import setup, find_packages


# Get the long description from the relevant file
with codecs_open('README.rst', encoding='utf-8') as f:
    long_description = f.read()


setup(name='usgs',
      version='0.1.2',
      description=u"Access the USGS inventory service",
      long_description=long_description,
      classifiers=[],
      keywords='',
      author=u"Amit Kapadia",
      author_email='amit@mapbox.com',
      url='https://github.com/mapbox/usgs',
      license='MIT',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      package_data={'usgs': ['data/datasets.json']},
      zip_safe=False,
      install_requires=['click', 'requests'],
      extras_require={
          'test': ['pytest'],
      },
      entry_points="""
      [console_scripts]
      usgs=usgs.scripts.cli:usgs
      """
      )



   
import os
from xml.etree import ElementTree
import requests

from usgs import USGS_API, USGSError, USGSConnectionError
from usgs import soap, xsi


TMPFILE = os.path.join("/", "tmp", "usgs")
NAMESPACES = {
    "SOAP-ENV": "http://schemas.xmlsoap.org/soap/envelope/",
    "ns1": "https://earthexplorer.usgs.gov/inventory/soap",
    "eemetadata": "http://earthexplorer.usgs.gov/eemetadata.xsd"
}

def _get_api_key():
    
    api_key = None
    
    if os.path.exists(TMPFILE):
        with open(TMPFILE, "r") as f:
            api_key = f.read()
    
    return api_key


def _check_for_usgs_error(root):
    fault_code_el = root.find("SOAP-ENV:Body/SOAP-ENV:Fault/faultcode", NAMESPACES)
    
    if fault_code_el is None:
        return
    
    fault_string_el = root.find("SOAP-ENV:Body/SOAP-ENV:Fault/faultstring", NAMESPACES)
    
    fault_code = fault_code_el.text
    fault_string = fault_string_el.text
    
    if fault_code == "AUTH_UNAUTHORIZED":
        raise USGSConnectionError(fault_string)
    else:
        raise USGSError(fault_string)


def clear_bulk_download_order():
    raise NotImplementedError

    
def clear_order():
    raise NotImplementedError
    

def datasets(dataset, node, ll=None, ur=None, start_date=None, end_date=None):
    
    api_key = _get_api_key()
    
    xml = soap.datasets(dataset, node, ll=ll, ur=ur, start_date=start_date, end_date=end_date, api_key=api_key)
    r = requests.post(USGS_API, xml)
    
    root = ElementTree.fromstring(r.text)
    _check_for_usgs_error(root)
    
    items = root.findall("SOAP-ENV:Body/ns1:datasetsResponse/return/item", NAMESPACES)
    
    data = map(lambda item: { el.tag: xsi.get(el) for el in item }, items)
    
    return data


def dataset_fields(dataset, node):
    
    api_key = _get_api_key()
    
    xml = soap.dataset_fields(dataset, node, api_key=api_key)
    r = requests.post(USGS_API, xml)
    
    root = ElementTree.fromstring(r.text)
    _check_for_usgs_error(root)
    
    items = root.findall("SOAP-ENV:Body/ns1:datasetFieldsResponse/return/item", NAMESPACES)
    data = map(lambda item: { el.tag: xsi.get(el) for el in item }, items)
    
    return data
    

def download(dataset, node, entityids, product):
    """
    Though USGS supports multiple products in a single request, there's
    ambiguity in the returned list. This wrapper only allows a single
    product per request.
    
    Additionally, the response has no indiction which URL is associated
    with which scene/entity id. The URL can be parsed, but the structure
    varies depending on the product.
    """
    
    api_key = _get_api_key()

    xml = soap.download(dataset, node, entityids, [product], api_key=api_key)
    r = requests.post(USGS_API, xml)
    
    root = ElementTree.fromstring(r.text)
    _check_for_usgs_error(root)
    
    items = root.findall("SOAP-ENV:Body/ns1:downloadResponse/return/item", NAMESPACES)
    
    data = map(lambda el: xsi.get(el), items)
    
    return data
    

def download_options(dataset, node, entityids):
    
    api_key = _get_api_key()
    
    xml = soap.download_options(dataset, node, entityids, api_key=api_key)
    r = requests.post(USGS_API, xml)
    
    root = ElementTree.fromstring(r.text)
    _check_for_usgs_error(root)
    
    items = root.findall("SOAP-ENV:Body/ns1:downloadOptionsResponse/return/item/downloadOptions/item", NAMESPACES)
    
    data = map(lambda item: { el.tag: xsi.get(el) for el in item }, items)
    
    return data


def get_bulk_download_products():
    raise NotImplementedError
    

def get_order_products():
    raise NotImplementedError
    

def hits():
    raise NotImplementedError
    

def item_basket():
    raise NotImplementedError
    
    
def login(username, password):
    xml = soap.login(username, password)
    r = requests.post(USGS_API, xml)
    
    if r.status_code is not 200:
        raise USGSError(r.text)

    root = ElementTree.fromstring(r.text)
    _check_for_usgs_error(root)
    
    element = root.find("SOAP-ENV:Body/ns1:loginResponse/return", NAMESPACES)
    
    api_key = element.text
    
    with open(TMPFILE, "w") as f:
        f.write(api_key)
    
    return api_key
    
    
def logout():
    
    api_key = _get_api_key()
    
    xml = soap.logout(api_key=api_key)
    r = requests.post(USGS_API, xml)
    
    if os.path.exists(TMPFILE):
        os.remove(TMPFILE)
    
    return True
    

def metadata(dataset, node, sceneids, extended=False, api_key=None):
    """
    Request metadata for a given scene in a USGS dataset.
    
    :param dataset:
    :param node:
    :param sceneids:
    :param extended:
        Send a second request to the metadata url to get extended metadata on the scene.
    :param api_key:
    """
    api_key = _get_api_key()
    
    xml = soap.metadata(dataset, node, sceneids, api_key=api_key)
    r = requests.post(USGS_API, xml)
    
    root = ElementTree.fromstring(r.text)
    _check_for_usgs_error(root)
    
    items = root.findall("SOAP-ENV:Body/ns1:metadataResponse/return/item", NAMESPACES)
    
    data = map(lambda item: { el.tag: xsi.get(el) for el in item }, items)
    
    if extended:
        
        def get_extended(scene):
            metadata_url = scene.get('metadataUrl')
            
            r = requests.get(metadata_url)
            root = ElementTree.fromstring(r.text)
            items = root.findall("eemetadata:metadataFields/eemetadata:metadataField", NAMESPACES)
            scene['extended'] = { item.attrib.get('name').strip(): xsi.get(item[0]) for item in items }
            
            return scene
        
        data = map(get_extended, data)
    
    return data
    

def remove_bulk_download_scene():
    raise NotImplementedError
    

def remove_order_scene():
    raise NotImplementedError
    

def search(dataset, node, lat=None, lng=None, distance=100, ll=None, ur=None, start_date=None, end_date=None, where=None, max_results=50000, starting_number=1, sort_order="DESC", api_key=None):
    """
    .. todo:: Export metadata from the search results e.g.
    
        <numberReturned xsi:type="xsd:int">41</numberReturned>
        <totalHits xsi:type="xsd:int">41</totalHits>
        <firstRecord xsi:type="xsd:int">1</firstRecord>
        <lastRecord xsi:type="xsd:int">41</lastRecord>
        <nextRecord xsi:type="xsd:int">41</nextRecord>
    """
    api_key = _get_api_key()
    
    xml = soap.search(dataset, node, lat=lat, lng=lng, distance=100, ll=ll, ur=ur, start_date=start_date, end_date=end_date, where=where, max_results=max_results, starting_number=starting_number, sort_order=sort_order, api_key=api_key)
    r = requests.post(USGS_API, xml)
    
    root = ElementTree.fromstring(r.text)
    _check_for_usgs_error(root)
    
    items = root.findall("SOAP-ENV:Body/ns1:searchResponse/return/results/item", NAMESPACES)
    
    data = map(lambda item: { el.tag: xsi.get(el) for el in item }, items)
    
    return data


def submit_bulk_order():
    raise NotImplementedError
    

def submit_order():
    raise NotImplementedError
    

def update_bulk_download_scene():
    raise NotImplementedError
    

def update_order_scene():
    raise NotImplementedError
    